// main.cpp
//
// ICS 65 Fall 2012
// Code Example
//
// A quick exploration of struct layout and padding.

#include <iostream>
using namespace std;



// A previous code example included this handy method, which takes two pointers
// and returns, as an integer number of bytes, how far the second pointer is
// away from the first.

int getOffset(void* basePointer, void* offsetPointer)
{
	return static_cast<int>(
		static_cast<char*>(offsetPointer) - static_cast<char*>(basePointer));
}



// Here is one struct, which I've intentionally set up to require padding
// (at least on our reference architecture).

struct S1
{
	char a;
	int b;
	short c;
	double d;
};



void layoutOfS1()
{
	cout << "S1 Layout" << endl;
	cout << "sizeof(S1): " << sizeof(S1) << endl;

	S1 s1;
	cout << "offset(S1.a): " << getOffset(&s1, &(s1.a)) << endl;
	cout << "offset(S1.b): " << getOffset(&s1, &(s1.b)) << endl;
	cout << "offset(S1.c): " << getOffset(&s1, &(s1.c)) << endl;
	cout << "offset(S1.d): " << getOffset(&s1, &(s1.d)) << endl;
	cout << endl;
}



// This is another struct, with the same four fields as S1 arranged in a
// different order.  Note that it is smaller and requires no padding
// (except padding added to the end of the struct, so that an array of
// structs will be aligned properly).

struct S2
{
	double d;
	int b;
	short c;
	char a;
};



void layoutOfS2()
{
	cout << "S2 Layout" << endl;
	cout << "sizeof(S2): " << sizeof(S2) << endl;

	S2 s2;
	cout << "offset(S2.d): " << getOffset(&s2, &(s2.d)) << endl;
	cout << "offset(S2.b): " << getOffset(&s2, &(s2.b)) << endl;
	cout << "offset(S2.c): " << getOffset(&s2, &(s2.c)) << endl;
	cout << "offset(S2.a): " << getOffset(&s2, &(s2.a)) << endl;
	cout << endl;
}



int main()
{
	layoutOfS1();
	layoutOfS2();

	return 0;
}
